--!parent
--!encoding=utf-8

--****************************************************************************
-- @file			LSW CORE LIB: FILESYSTEM
-- @author		Valentin Schmidt
-- @version	 0.3
-- @requires	xtra("BinFile")
--****************************************************************************

property $

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	$.loadXtras(me.libdir, ["BinFile.x32"], ["BinFile.xtra"])
	
	--put xtra("BinFile").interface()
	
	-- win only
	me._loadFileAttributeConstants()
	me._loadOpenFileNameFlags()
	me._loadFileAttributeConstants()
	
	return me
end

-- files

----------------------------------------
-- Copies file
-- @param {string} fromFn
-- @param {string} toFn
-- @return {bool}
----------------------------------------
on fileCopy (me, fromFn, toFn)
	return bx_file_copy(fromFn, toFn)
end

----------------------------------------
-- Deletes file
-- @param {string} fn
-- @return {bool}
----------------------------------------
on fileDelete (me, fn)
	return bx_file_delete(fn)
end

----------------------------------------
-- Checks if file exists
-- @param {string} fn
-- @return {bool}
----------------------------------------
on fileExists (me, fn)
	return (bx_exists(fn)=1)
end

----------------------------------------
-- Moves file
-- @param {string} fn
-- @return {bool}
----------------------------------------
on fileMove (me, fromFn, toFn)
	return bx_rename(fromFn, toFn)
end

----------------------------------------
-- Recycles file
-- @param {string} fn
-- @return {bool}
----------------------------------------
on fileRecycle (me, fn)
	return bx_file_recycle(fn)
end

----------------------------------------
-- Renames file
-- @param {string} fromFn
-- @param {string} toFn
-- @return {bool}
----------------------------------------
on fileRename (me, fromFn, toFn)
	return bx_rename(fromFn, toFn)
end

----------------------------------------
-- Returns file size
-- @param {string} fn
-- @return {integer}
----------------------------------------
on fileSize (me, fn)
	return bx_file_size(fn)
end

----------------------------------------
-- Returns file size
-- @param {string} fn
-- @return
----------------------------------------
on fileDate (me, fn)
	return bx_file_date(fn)
end

----------------------------------------
-- Truncates file
-- @param {string} fn
-- @param {integer} size
-- @return {bool}
----------------------------------------
on fileTruncate (me, fn, size)
	return bx_file_truncate(fn, size)
end

-- folders

----------------------------------------
-- Copies folder
-- @param {string} fromFn
-- @param {string} toFn
-- @return {bool}
----------------------------------------
on folderCopy (me, fromFn, toFn)
	return bx_folder_copy(fromFn, toFn)
end

----------------------------------------
-- Creates folder
-- @param {string} foldername
-- @return {bool}
----------------------------------------
on folderCreate (me, foldername)
	return bx_folder_create(foldername)
end

----------------------------------------
-- Deletes folder
-- @param {string} foldername
-- @return {bool}
----------------------------------------
on folderDelete (me, foldername)
	return bx_folder_delete(foldername)
end

----------------------------------------
-- Deletes folder including all contents - USAGE ON YOUR OWN RISK
-- @param {string} foldername
-- @return {bool}
----------------------------------------
on folderDeleteRecursive (me, foldername)
	if the last char of foldername="\" then delete the last char of foldername
	res = me.folderList(foldername)	
	repeat with f in res
		if the last char of f="\" then
			ok = me.folderDeleteRecursive(foldername&"\"&f)
		else
			ok = me.fileDelete(foldername&"\"&f)
		end if
		if not ok then return FALSE
	end repeat
	return me.folderDelete(foldername)
end

----------------------------------------
-- Checks if folder exists
-- @param {string} fn
-- @return {bool}
----------------------------------------
on folderExists (me, foldername)
	return (bx_exists(foldername)=2)
end

----------------------------------------
-- Returns list of files and folders
-- @param {string} folderName
-- @return {list}
----------------------------------------
on folderList (me, folderName)
	return bx_folder_get_contents(folderName)
end

----------------------------------------
-- Renames folder
-- @param {string} fromFn
-- @param {string} toFn
-- @return {bool}
----------------------------------------
on folderRename (me, fromFn, toFn)
	return bx_rename(fromFn, toFn)
end

----------------------------------------
-- Moves folder (only within same volume)
-- @param {string} fn
-- @return {bool}
----------------------------------------
on folderMove (me, fromFn, toFn)
	return bx_rename(fromFn, toFn)
end

----------------------------------------
-- File Attribute Constants
-- https://msdn.microsoft.com/en-us/library/windows/desktop/gg258117(v=vs.85).aspx
----------------------------------------
on _loadFileAttributeConstants (me)
	me[#FILE_ATTRIBUTE_ARCHIVE] = 32
	me[#FILE_ATTRIBUTE_HIDDEN] = 2
	me[#FILE_ATTRIBUTE_NORMAL] = 128
	me[#FILE_ATTRIBUTE_NOT_CONTENT_INDEXED] = 8192
	me[#FILE_ATTRIBUTE_OFFLINE] = 4096
	me[#FILE_ATTRIBUTE_READONLY] = 1
	me[#FILE_ATTRIBUTE_SYSTEM] = 4
	me[#FILE_ATTRIBUTE_TEMPORARY] = 256
end

----------------------------------------
-- OPENFILENAME Flags
-- https://msdn.microsoft.com/en-us/library/windows/desktop/ms646839(v=vs.85).aspx
----------------------------------------
on _loadOpenFileNameFlags (me)
	me[#OFN_READONLY]								 = 1
	me[#OFN_OVERWRITEPROMPT]					= 2
	me[#OFN_HIDEREADONLY]						 = 4
	me[#OFN_NOCHANGEDIR]							= 8
	me[#OFN_SHOWHELP]								 = 16
	me[#OFN_ENABLEHOOK]							 = 32
	me[#OFN_ENABLETEMPLATE]					 = 64
	me[#OFN_ENABLETEMPLATEHANDLE]		 = 128
	me[#OFN_NOVALIDATE]							 = 256
	me[#OFN_ALLOWMULTISELECT]				 = 512
	me[#OFN_EXTENSIONDIFFERENT]			 = 1024
	me[#OFN_PATHMUSTEXIST]						= 2048
	me[#OFN_FILEMUSTEXIST]						= 4096
	me[#OFN_CREATEPROMPT]						 = 8192
	me[#OFN_SHAREAWARE]							 = 16384
	me[#OFN_NOREADONLYRETURN]				 = 32768
	me[#OFN_NOTESTFILECREATE]				 = 65536
	me[#OFN_NONETWORKBUTTON]					= 131072
	me[#OFN_NOLONGNAMES]							= 262144
	me[#OFN_EXPLORER]								 = 524288
	me[#OFN_NODEREFERENCELINKS]			 = 1048576
	me[#OFN_LONGNAMES]								= 2097152
	me[#OFN_ENABLEINCLUDENOTIFY]			= 4194304
	me[#OFN_ENABLESIZING]						 = 8388608
end

----------------------------------------
-- 
----------------------------------------
on _loadOpenFileDisplayModes (me)
	me[#ODM_VIEW_ICONS]	= 28713
	me[#ODM_VIEW_LIST]	 = 28715
	me[#ODM_VIEW_DETAIL] = 28716
	me[#ODM_VIEW_THUMBS] = 28717
	me[#ODM_VIEW_TILES]	= 28718
end
